﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using OpenStack.Swift;
using Rackspace.Cloudfiles;

namespace Ucloudbiz.Storage
{
    /// <summary>
    /// KT_Container object that provides Container level functionality and information through CF_Container
    /// Added fucntions : MakePublicRead(), MakePrivateRead()
    /// </summary>
    public class KT_Container : CF_Container
    {
        private string _name = null;
        private int _retires = 0;
        private int _num_retries_attempted = 0;
        private Rackspace.Cloudfiles.Client _client;
        private Connection _conn;
        /// <summary>
        /// Initializes a new instance of the <see cref="Ucloudbiz.Storage.KT_Container"/> class.
        /// </summary>
        /// <param name='conn'>
        /// Conn.
        /// </param>
        /// <param name='client'>
        /// Client.
        /// </param>
        /// <param name='container_name'>
        /// Container_name.
        /// </param>
        public KT_Container(Connection conn, Rackspace.Cloudfiles.Client client, string container_name)
            : base(conn, client, container_name)
        {
            Common.ValidateContainerName(container_name);
            _client = client;
            _conn = conn;
            _name = container_name;
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="Ucloudbiz.Storage.KT_Container"/> class.
        /// </summary>
        /// <param name='conn'>
        /// Conn.
        /// </param>
        /// <param name='container_name'>
        /// Container_name.
        /// </param>
        public KT_Container(Connection conn, string container_name)
            : base(conn, container_name)
        {
            Common.ValidateContainerName(container_name);
            _client = new KT_Client();
            _conn = conn;
            _name = container_name;
        }

        /// <summary>
        /// Makes the container public.
        /// </summary>
        public void MakePublicRead()
        {
            _client.Timeout = _conn.Timeout;
            var headers = new Dictionary<string, string>
                {
                    {Rackspace.Cloudfiles.Constants.Headers.UserAgent, _conn.UserAgent},
                    {KT_Constants.Headers.ReadAuthority, ".r:*"}
                };
            try
            {
                _client.PostContainer(_conn.UserCreds.StorageUrl.ToString(), _conn.UserCreds.AuthToken, Name, headers, new Dictionary<string, string>());
            }
            catch (ClientException e)
            {
                switch (e.Status)
                {
                    case Rackspace.Cloudfiles.Constants.StatusCodes.Timeout:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            MakePublicRead();
                            break;
                        }
                        else
                        {
                            throw new TimeoutException();
                        }
                    case Rackspace.Cloudfiles.Constants.StatusCodes.Unauthorized:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            _conn.Authenticate();
                            MakePublicRead();
                            break;
                        }
                        else
                        {
                            throw new UnauthorizedException();
                        }
                    default:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            MakePublicRead();
                            break;
                        }
                        else
                        {
                            throw new CloudFilesException("Error: " + e.Status.ToString(CultureInfo.InvariantCulture));
                        }
                }
            }
            finally
            {
                _num_retries_attempted = 0;
            }
        }
        /// <summary>
        /// Makes the Container private.
        /// </summary>
        public void MakePrivateRead()
        {
            _client.Timeout = _conn.Timeout;
            var headers = new Dictionary<string, string>
			                  {
			                      {Rackspace.Cloudfiles.Constants.Headers.UserAgent, _conn.UserAgent},
                                  {KT_Constants.Headers.ReadAuthority, ".r:"+_conn.UserCreds.StorageUrl.ToString()}
			                  };
            try
            {
                _client.PostContainer(_conn.UserCreds.StorageUrl.ToString(), _conn.UserCreds.AuthToken.ToString(), Name, headers, new Dictionary<string, string>());
            }
            catch (ClientException e)
            {
                switch (e.Status)
                {
                    case Rackspace.Cloudfiles.Constants.StatusCodes.Timeout:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            MakePrivateRead();
                            break;
                        }
                        else
                        {
                            throw new TimeoutException();
                        }
                    case Rackspace.Cloudfiles.Constants.StatusCodes.Unauthorized:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            _conn.Authenticate();
                            MakePrivateRead();
                            break;
                        }
                        else
                        {
                            throw new UnauthorizedException();
                        }
                    case Rackspace.Cloudfiles.Constants.StatusCodes.NotFound:
                        throw new ContainerNotFoundException();
                    default:
                        if (_num_retries_attempted < _retires)
                        {
                            ++_num_retries_attempted;
                            MakePrivateRead();
                            break;
                        }
                        else
                        {
                            throw new CloudFilesException("Error: " + e.Status.ToString(CultureInfo.InvariantCulture));
                        }
                }
            }
            finally
            {
                _num_retries_attempted = 0;
            }
        }
    }
}
